@@ -33,15 +33,10 @@
 #include <ctype.h>
 
 #include <glib/gi18n.h>
-#include <gnome.h>
-
-#include <gmenu-tree.h>
 
 #include <bonobo/bonobo-exception.h>
 #include <bonobo-activation/bonobo-activation.h>
 
-#include <dirent.h>
-
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 #include <libxml/xmlmemory.h>
@@ -232,110 +227,140 @@ NULL)) {
 	CORBA_free (info_list);
 }
 
-static int
-compare_applications (GMenuTreeEntry *a,
-		      GMenuTreeEntry *b)
+static void
+add_bugzilla_application_from_desktop_file (GHashTable *hash,
+					    const char *desktop_file_path)
 {
-	return g_utf8_collate (gmenu_tree_entry_get_name (a),
-			       gmenu_tree_entry_get_name (b));
-}
+	GKeyFile *key_file;
+	GError	 *error = NULL;
+	char *name;
+	char *cname;
+	char *comment;
+	char *bugzilla;
+	char *product;
+	char *component;
+	char *version;
+	char *icon;
+	char *exec;
+	char *other_binaries;
+	char *extra_info_script;
 
-static GSList *get_all_applications_from_dir (GMenuTreeDirectory *directory,
-					      GSList             *list);
-
-static GSList *
-get_all_applications_from_alias (GMenuTreeAlias *alias,
-				 GSList         *list)
-{
-	GMenuTreeItem *aliased_item;
-
-	aliased_item = gmenu_tree_alias_get_item (alias);
-
-	switch (gmenu_tree_item_get_type (aliased_item)) {
-	case GMENU_TREE_ITEM_DIRECTORY:
-		list = get_all_applications_from_dir (GMENU_TREE_DIRECTORY (aliased_item), list);
-		break;
-
-	case GMENU_TREE_ITEM_ENTRY:
-		list = g_slist_append (list, gmenu_tree_item_ref (aliased_item));
-		break;
-
-	default:
-		break;
+	key_file = g_key_file_new ();
+	g_key_file_load_from_file (key_file, desktop_file_path,
+				   G_KEY_FILE_NONE, &error);
+	if (error) {
+		g_warning ("Couldn't load %s: %s", desktop_file_path,
+			   error->message);
+		g_error_free (error);
+		error = NULL;
+		return;
 	}
 
-	gmenu_tree_item_unref (aliased_item);
-
-	return list;
-}
-
-static GSList *
-get_all_applications_from_dir (GMenuTreeDirectory *directory,
-			       GSList             *list)
-{
-	GSList *items;
-	GSList *l;
-
-	if (g_main_context_pending (NULL)) {
-		g_main_context_iteration (NULL, FALSE);
+	if (!g_key_file_has_group (key_file, DESKTOP_ENTRY) ||
+	    !g_key_file_has_key (key_file, DESKTOP_ENTRY, BUGZILLA_BUGZILLA, &error)) {
+		g_key_file_free (key_file);	
+		if (error)
+			g_error_free (error);
+		return;
 	}
 
-	items = gmenu_tree_directory_get_contents (directory);
-	for (l = items; l; l = l->next) {
-		GMenuTreeItem *item = l->data;
+	name = g_key_file_get_locale_string (key_file, DESKTOP_ENTRY, DESKTOP_NAME, NULL, NULL);
+	cname = g_key_file_get_string (key_file, DESKTOP_ENTRY, DESKTOP_NAME, NULL);
+	comment = g_key_file_get_locale_string (key_file, DESKTOP_ENTRY, DESKTOP_COMMENT, NULL, NULL);
+	bugzilla = g_key_file_get_string (key_file, DESKTOP_ENTRY, BUGZILLA_BUGZILLA, NULL);
+	product = g_key_file_get_string (key_file, DESKTOP_ENTRY, BUGZILLA_PRODUCT, NULL);
+	component = g_key_file_get_string (key_file, DESKTOP_ENTRY, BUGZILLA_COMPONENT, NULL);
+	version = g_key_file_get_string (key_file, DESKTOP_ENTRY, BUGZILLA_VERSION, NULL);
+	icon = g_key_file_get_string (key_file, DESKTOP_ENTRY, DESKTOP_ICON, NULL);
+	exec = g_key_file_get_string (key_file, DESKTOP_ENTRY, DESKTOP_EXEC, NULL);
+	other_binaries = g_key_file_get_string (key_file, DESKTOP_ENTRY, BUGZILLA_OTHER_BINARIES, NULL);
+	extra_info_script = g_key_file_get_string (key_file, DESKTOP_ENTRY, BUGZILLA_EXTRA_INFO_SCRIPT, NULL);
 
-		switch (gmenu_tree_item_get_type (item)) {
-		case GMENU_TREE_ITEM_DIRECTORY:
-			list = get_all_applications_from_dir (GMENU_TREE_DIRECTORY (item), list);
-			break;
+	add_bugzilla_application (hash,
+				  name,
+				  cname,
+				  comment,
+				  bugzilla,
+				  product,
+				  component,
+				  version,
+				  icon,
+				  exec,
+				  other_binaries,
+				  extra_info_script);
+	g_free (name);
+	g_free (cname);
+	g_free (comment);
+	g_free (bugzilla);
+	g_free (product);
+	g_free (component);
+	g_free (version);
+	g_free (icon);
+	g_free (exec);
+	g_free (other_binaries);
+	g_free (extra_info_script);
+	g_key_file_free (key_file);
+}
 
-		case GMENU_TREE_ITEM_ENTRY:
-			list = g_slist_append (list, gmenu_tree_item_ref (item));
-			break;
+static void
+load_applications_from_dir (GHashTable *hash,
+			    const char *path)
+{
+	GDir *dir;
+	char *name;
+	char *desktop_file;
 
-		case GMENU_TREE_ITEM_ALIAS:
-			list = get_all_applications_from_alias (GMENU_TREE_ALIAS (item), list);
-			break;
+	dir = g_dir_open (path, 0, NULL);
+	if (!dir)
+		return;
 
-		default:
-			break;
+	while ((name = g_dir_read_name (dir))) {
+		if (g_main_context_pending (NULL)) {
+			g_main_context_iteration (NULL, FALSE);
 		}
 
-		gmenu_tree_item_unref (item);
+		if (!g_str_has_suffix (name, ".desktop")) {
+			continue;
+		}
+
+		desktop_file = g_build_filename (path, name, NULL);
+		add_bugzilla_application_from_desktop_file (hash, desktop_file);
+		g_free (desktop_file);
 	}
 
-	g_slist_free (items);
-
-	return list;
+	g_dir_close (dir);
 }
 
-static GSList *
-get_all_applications (void)
+static char **
+get_all_directories (void)
 {
-	GMenuTree          *tree;
-	GMenuTreeDirectory *root;
-	GSList             *retval;
-	const char         *menufile = BUDDY_DATADIR "/bug-buddy.menu";
+	GPtrArray          *dirs;
+	const char * const *system_config_dirs;
+	const char * const *system_data_dirs;
+	int                 i;
 
-	if (g_file_test (menufile, G_FILE_TEST_IS_REGULAR)) {
-		/* use a custom menu file to scan desktop entry files so we aren't limited
-		 * to reporting bugs only present in the applications menu*/
-		tree = gmenu_tree_lookup (menufile, GMENU_TREE_FLAGS_INCLUDE_NODISPLAY);
-	} else {
-		/* fallback to using the applications menu */
-		tree = gmenu_tree_lookup ("applications.menu", GMENU_TREE_FLAGS_INCLUDE_NODISPLAY);
+	dirs = g_ptr_array_new ();
+
+	system_data_dirs = g_get_system_data_dirs ();
+	for (i = 0; system_data_dirs[i]; i++) {
+		g_ptr_array_add (dirs,
+				 g_build_filename (system_data_dirs[i],
+						   "applications", NULL));
+		g_ptr_array_add (dirs,
+				 g_build_filename (system_data_dirs[i],
+						   "gnome", "autostart", NULL));
 	}
 
-	root = gmenu_tree_get_root_directory (tree);
+	system_config_dirs = g_get_system_config_dirs ();
+	for (i = 0; system_config_dirs[i]; i++) {
+		g_ptr_array_add (dirs,
+				 g_build_filename (system_config_dirs[i],
+						   "autostart", NULL));
+	}
 
-	retval = get_all_applications_from_dir (root, NULL);
+	g_ptr_array_add (dirs, NULL);
 
-	gmenu_tree_item_unref (root);
-	gmenu_tree_unref (tree);
-
-	retval = g_slist_sort (retval, (GCompareFunc) compare_applications);
-
-	return retval;
+	return (char **) g_ptr_array_free (dirs, FALSE);
 }
 
 GQuark
@@ -347,98 +372,15 @@ bugzilla_error_quark (void)
 GHashTable *
 load_applications (void)
 {
-	GSList         *all_applications;
-	GSList         *l;
-	char           *prev_name = NULL;
-	GError	       *error = NULL;
+	char **directories;
+	int    i;
 
 	GHashTable *program_to_application = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) application_free);
 
-	all_applications = get_all_applications ();
-	for (l = all_applications; l; l = l->next) {
-		GKeyFile *key_file;
-		char *name;
-		char *cname;
-		char *comment;
-		char *bugzilla;
-		char *product;
-		char *component;
-		char *version;
-		char *icon;
-		char *exec;
-		char *other_binaries;
-		char *extra_info_script;
-		GMenuTreeEntry *entry = l->data;
-
-		if (g_main_context_pending (NULL)) {
-			g_main_context_iteration (NULL, FALSE);
-		}
-
-		if (prev_name && strcmp (gmenu_tree_entry_get_name (entry), prev_name) == 0) {
-			gmenu_tree_item_unref (entry);
-			continue;
-		}
-		key_file = g_key_file_new ();
-		g_key_file_load_from_file (key_file, gmenu_tree_entry_get_desktop_file_path (entry), 
-					   G_KEY_FILE_NONE, &error);
-		if (error) {
-			g_warning ("Couldn't load %s: %s", gmenu_tree_entry_get_desktop_file_path (entry),
-				   error->message);
-			g_error_free (error);
-			error = NULL;
-			gmenu_tree_item_unref (entry);
-			continue;
-		}
-    
-		if (!g_key_file_has_group (key_file, DESKTOP_ENTRY) || !g_key_file_has_key (key_file, DESKTOP_ENTRY, BUGZILLA_BUGZILLA, &error)) {
-			g_key_file_free (key_file);	
-			gmenu_tree_item_unref (entry);
-			if (error)
-				g_error_free (error);
-			continue;
-		}
-
-		name = g_key_file_get_locale_string (key_file, DESKTOP_ENTRY, DESKTOP_NAME, NULL, NULL);
-		cname = g_key_file_get_string (key_file, DESKTOP_ENTRY, DESKTOP_NAME, NULL);
-		comment = g_key_file_get_locale_string (key_file, DESKTOP_ENTRY, DESKTOP_COMMENT, NULL, NULL);
-		bugzilla = g_key_file_get_string (key_file, DESKTOP_ENTRY, BUGZILLA_BUGZILLA, NULL);
-		product = g_key_file_get_string (key_file, DESKTOP_ENTRY, BUGZILLA_PRODUCT, NULL);
-		component = g_key_file_get_string (key_file, DESKTOP_ENTRY, BUGZILLA_COMPONENT, NULL);
-		version = g_key_file_get_string (key_file, DESKTOP_ENTRY, BUGZILLA_VERSION, NULL);
-		icon = g_key_file_get_string (key_file, DESKTOP_ENTRY, DESKTOP_ICON, NULL);
-		exec = g_key_file_get_string (key_file, DESKTOP_ENTRY, DESKTOP_EXEC, NULL);
-		other_binaries = g_key_file_get_string (key_file, DESKTOP_ENTRY, BUGZILLA_OTHER_BINARIES, NULL);
-		extra_info_script = g_key_file_get_string (key_file, DESKTOP_ENTRY, BUGZILLA_EXTRA_INFO_SCRIPT, NULL);
-
-		add_bugzilla_application (program_to_application,
-					  name,
-					  cname,
-					  comment,
-					  bugzilla,
-					  product,
-					  component,
-					  version,
-					  icon,
-					  exec,
-					  other_binaries,
-					  extra_info_script);
-		g_free (name);
-		g_free (cname);
-		g_free (comment);
-		g_free (bugzilla);
-		g_free (product);
-		g_free (component);
-		g_free (version);
-		g_free (icon);
-		g_free (exec);
-		g_free (other_binaries);
-		g_free (extra_info_script);
-		g_free (prev_name);
-		prev_name = g_strdup (gmenu_tree_entry_get_name (entry));
-		g_key_file_free (key_file);	
-		gmenu_tree_item_unref (entry);
-	}
-	g_slist_free (all_applications);
+	directories = get_all_directories ();
+	for (i = 0; directories[i] != NULL; i++)
+		load_applications_from_dir (program_to_application, directories[i]);
+	g_strfreev (directories);
 
 	load_applets (program_to_application);
 
